class JobStates(object):
    ENQUEUED="crowdai.job_state.ENQUEUED"
    WAITING="crowdai.job_state.WAITING"
    RUNNING="crowdai.job_state.RUNNING"
    PROGRESS_UPDATE="crowdai.job_state.PROGRESS_UPDATE"
    COMPLETE="crowdai.job_state.COMPLETE"
    ERROR="crowdai.job_state.ERROR"
    INFO="crowdai.job_state.INFO"
    TIMEOUT="crowdai.job_state.TIMEOUT"
