class CrowdAIResponseEvent(object):
    SUCCESS = "crowdAI.Event.SUCCESS"
    ERROR = "crowdAI.Event.ERROR"
    WARNING = "crowdAI.Event.WARNING"
    INFO = "crowdAI.Event.INFO"
    DEBUG = "crowdAI.Event.DEBUG"
